
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% This main code is to control and run all other functions. 
%
%This program is to calculate the active layer thickness ALT and
%Topographical wetness index TWI and drainage path from all cells in a DEM.
%Drainage area to each cell also the slope and aspect
%
% 1- the program will use reading function to read the DEM file by input
% The file name with full path. This is required to run this function.
% Also the format of the input file should be ESRI (ASCII) file & 
% (file_name.asc). Please read the word text file for more details.
% Except for the border cells all the other cells should has data value this
% mean Nan is not allowed (No NaN cells).
% As a result from the reading function we will have a matrix of elevations 
% data(ZT), number of rows (nrows),number of columns (ncols), The coordinates
% of the bottom left corner (xllcorner,yllcorner), DEM cell size (cellsize)
% and the nodata value (NODATA_value).
%
% 2- The main program will use classification function to classify all cells to 
% five different classes (Complicated, sink, flat, peak and undisturbed)
%
% 3- The main program will use flowDistribution function to assign the flow path from
% all cells to their 8 neighbor cells. this is TFM algorithm
% this will result 8 matrix showing drainage paths from each cell to 8 possible directions
% This function will also use the drainage path to calculate the drainage
% area of each cell
% 
% 4- function named TWIALT will be used to calculate the active layer
% thickness ALT and topographical wittness index TWI
%
% 5- track_upstream function, In this function the flow is tracked up. 
% at first we need to flag a cell by giving the row and column number for
% the target cell. % the function will creat new grid with 1 in all cells that has flow to the
% target cell. all other cells will get the value of 0
% 
% 6- The main program will Use writing function to write the DEM to an output 
% file.
%Abdulghani 2009-11-26
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1-
%% clear
clear
%%%
% Function for reading DEM file
sprintf('write the full path and name of the input file Ex:')
input_file_path=input(' (C:\Tempdata\DEMs\dem_idw_10_1.asc)=> ','S');
[ncols,nrows,xllcorner,yllcorner,cellsize,NODATA_value,Z]=Reading(input_file_path);
%
ZI = reshape(Z,ncols,nrows); %reshape the vector data to 2D matrix data
%Transpose the matrix, since we did read the text file from top while the 
%numbering starts from left bottom
ZT = ZI'; %transpose
clear Z
clear ZI
ZT(ZT==NODATA_value)= nan; %All no data are NaN
Mn=nanmin(nanmin(ZT));
INAN=isnan(ZT);
if Mn<=0
    %INAN=find(isnan(ZT));
        ZT(INAN)=NODATA_value;
             ZT=ZT + abs(Mn-1);
      ZT(INAN)=nan;       
end
clear INAN

%%%

%Using classification matrix to classify all cells to 5 groups
[Classification_matrix]=Classification(ZT,nrows,ncols);
%display number of cells in each class 
CLASSES = 1:5;
[numTimesInMatrix, Number] = hist(Classification_matrix(:),CLASSES);
Complicated_cells_id1=numTimesInMatrix(1)
Sink_cells_id2=numTimesInMatrix(2)
Flat_cells_id3=numTimesInMatrix(3)
Peak_cells_id4=numTimesInMatrix(4)
Undisturbed_cells_id5=numTimesInMatrix(5)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Force the flow between two points (Culverts)
sprintf('Do you want to connect the flow in culverts?')
YesNo1=input(' This will change the DEM, if Yes input 1 or skip it by entering any other number = ');
if YesNo1 ==1
[ZT]=culverts(ZT,nrows,ncols,cellsize,xllcorner,yllcorner);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%%% Remove sinks
sprintf('Do you want to fill/remove pits, sinks or topographic depressions?')
YesNo2=input(' This will change the DEM, for Yes input 1 = ');
if YesNo2 ==1
sprintf('Do you want to remove all sinks or just fill sinks with max specified depth?')
ALLorMAX=input(' To remove all sinks input 1 = ');
ZT_unfilled_sinks=ZT;
if ALLorMAX==1
[ZT,maxfd] = fillsinksagy(ZT);
%SB=watershed(ZT);
%SBfs=watershed(ZTfs);
end

if ALLorMAX ~=1
    maxdep=input('input the max depth of sinks to be filled, Max. depth in m = ');
[ZT,maxfd]= fillsinksAH(ZT,maxdep);
end
The_max_filled_depth_was = maxfd

end



 %%% ReUse the classification matrix to classify all cells to 5 groups
[Classification_matrix]=Classification(ZT,nrows,ncols);
%display number of cells in each class 
CLASSES = 1:5;
[numTimesInMatrix, Number] = hist(Classification_matrix(:),CLASSES);
Complicated_cells_id1=numTimesInMatrix(1)
Sink_cells_id2=numTimesInMatrix(2)
Flat_cells_id3=numTimesInMatrix(3)
Peak_cells_id4=numTimesInMatrix(4)
Undisturbed_cells_id5=numTimesInMatrix(5)
%end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sprintf('Next will distribute the flow from each cell, press enter')
pause
%Flow directions TFM
%using FlowPercent function to Calculate
% the flow percentage from each cell to the 8 neighboring cells
%The result is 8 files for 8 directions.
%also a check matrix to mark the cells with completed calculations

%x=input('input a value for the equation exponent x, default is one-- x= ');
[FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,Check_Matrix,INan]=FlowDistribution(Classification_matrix,ZT,nrows,ncols);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sprintf('To calculate the FLAT cells press enter')
pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

%Till here the calculation of the drainage path for 4 different cell forms
%is finished. now we will start new calculations for flat cells
%deal with flat flow function
% FLAT FLOW OUT
% if Classification_matrix(r,c)==3  %flat to skip now 999999999
[FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,Check_Matrix]=FLAT_Flow_out(FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,Classification_matrix,Check_Matrix,ZT,nrows,ncols);


sprintf('To calculate the FLAT cells flow to sinks if any, press enter')

pause
% for r=2:nrows-1
% for c=2:ncols-1  
% FLAT FLOW IN
% if Classification_matrix(r,c)==3
[FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,Check_Matrix]=FLAT_Flow_in(FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,Check_Matrix,ZT,Classification_matrix,nrows,ncols);
% end
clear Check_Matrix
% end %end for cols
% end %end for rows

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%calculate slope and Aspect
sprintf('To calculate the Slope and Aspect, press enter')
pause
[Slope,Aspect]=SlopeAspect(ZT,cellsize,nrows,ncols);

%Drainage area
sprintf('to calculate the DA')
pause

[DA,SDA,DA_Checked]=Drainage_Area(FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,INan,ZT,nrows,ncols,cellsize);
%Plot the DA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% To track the flow in the upstream direction
sprintf('To track the flow in the upstream direction')

target_r=input(' input the row number of the target cell = ');
target_c=input(' input the column number of the target cell = ');

[zz,trackup]=track_upstream(FP1,FP2,FP3,FP4,FP5,FP6,FP7,FP8,INan,nrows,ncols,target_r,target_c);
Number_of_cell_flowing_to_the_target_cell=zz
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% %function to compute Topographical wetness index TWI & Active layer
% %thickness ALT 
% [TWI,ALT]=TWIALT(Classification_matrix,DA,nrows,ncols,cellsize,Slope,Tds);
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%writing the DEM
%In the next line you will be asked to if you want to write the data to a file.
%also you will be asked to input the required matrix name that you want to
%write
yes =1;
while yes ==1
sprintf('write the name of the matrix you want to output to a file ex:')
THEMATRIX=input('For example, if you want to save the drainage area write DA =');
sprintf('write the full path and name of the output file Ex:')
output_file_path=input(' (C:\Tempdata\DEMs\Facc_MATLAB_10_1.asc)=> ','S');
[done]=Writing(output_file_path,ncols,nrows,xllcorner,yllcorner,cellsize,NODATA_value,THEMATRIX)
yes=input('Do you want to write another file? for yes input 1 for no input 2 = ')
end
%::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


